TICKER_KEY = 'Ticker'
TICKER_KEY_CAPITAL = 'TICKER'
COMPANY_KEY_CAPITAL = 'COMPANY'
DAYS_TO_EARNINGS_KEY_CAPITAL='DAYS_LEFT'
DAYS_TO_EARNINGS_KEY_BEFORE_FORMAT='days_left'

MARKET_DATA_TICKERS = ['^GSPC', '^TNX', '^RUT', '^VIX']
TBILL_10_YEAR = '^TNX'

FINVIZ_EARNINGS_DATE_KEY = 'EARNINGS_DATE'
DAYS_LEFT_KEY = 'DAYS_LEFT'
ALLOWED_CAPITALIZATIONS = ['micro', 'small', 'medium']

# Post processing fields 
ABS_RET_KEY='1+{}_RET'
EXP_RET_KEY='EXP.RET_{}'
BETA_KEY='BETA'
RF_KEY='1+{}_RF'
SP_500_TICKER='^GSPC'
MARK_EXP_KEY='MARK_EXP_{}'
CAPM_KEY='CAPM_{}'
TBILL_10_YEAR='^TNX'
VIX_TICKER='^VIX'
VIX_KEY='1+{}_VIX'
EARNING_VIX_KEY='EARNING_VIX'
CAR_KEY='CAR_{}'
BHAR_KEY='BHAR_{}'
DATADATE_KEY='DATADATE'
ALLOWED_WINDOWS = [3, 30, 60]

AVAILABLE_CHECK_COLUMNS = [
    EXP_RET_KEY,
    ABS_RET_KEY,
    MARK_EXP_KEY,
    CAPM_KEY,
    CAR_KEY,
    BHAR_KEY,
]

FIELDS_ORDER = [
'DAYS_LEFT',
'TICKER',
'COMPANY',
'PRICE',
'52W_NORM',
'AVG_VOLUME',
'DIVIDEND_EX_DATE',
'DIVIDEND',
'PAYOUT_RATIO',
'SECTOR',
'INDUSTRY',
'COUNTRY',
'MARKET_CAP',
'P/E',
'FWD_P/E',
'PEG',
'P/S',
'P/B',
'P/C',
'P/FCF',
'EPS',
'EPS_NEXT_Q',
'EPS_THIS_Y',
'EPS_NEXT_Y',
'EPS_PAST_5Y',
'EPS_NEXT_5Y',
'SALES_PAST_5Y',
'SALES_Q/Q',
'SALES_YOY_TTM',
'EPS_Q/Q',
'SALES',
'INCOME',
'EPS_SURPRISE',
'REVENUE_SURPRISE',
'OUTSTANDING',
'VOLUME',
'REL_VOLUME',
'FLOAT',
'FLOAT_%',
'INSIDER_OWN',
'INSIDER_TRANS',
'INST_OWN',
'INST_TRANS',
'SHORT_FLOAT',
'SHORT_RATIO',
'SHORT_INTEREST',
'ROA',
'ROE',
'ROIC',
'CURR_R',
'QUICK_R',
'LTDEBT/EQ',
'DEBT/EQ',
'GROSS_M',
'OPER_M',
'PROFIT_M',
'PERF_WEEK',
'PERF_MONTH',
'PERF_QUART',
'PERF_HALF',
'PERF_YEAR',
'PERF_YTD',
'BETA',
'ATR',
'VOLATILITY_W',
'VOLATILITY_M',
'SMA20',
'SMA50',
'SMA200',
'RSI',
'TARGET_PRICE',
'BOOK/SH',
'CASH/SH',
'EMPLOYEES',
'OPTIONABLE',
'PREV_CLOSE',
'SHORTABLE',
'RECOM',
'CHANGE',
'DIVIDEND_TTM',
'EPS_YOY_TTM',
'IS_S&P500',
'IS_RUSSELL',
'IS_NASDAQ',
'IS_DOW_JONES',
'IS_AMC',
'IS_BMO',
'IS_USA',
'DATADATE'
]